--- nnir.el~	Sun Sep 23 11:30:13 2001
+++ nnir.el	Sun Sep 23 10:55:45 2001
@@ -461,6 +461,11 @@ (defcustom nnir-excite-remove-prefix (co
   :type '(regexp)
   :group 'nnir)
 
+(defcustom nnir-imap-search-charset nil
+  "*Name of the charset of the strings that appear in the search criteria."
+  :type '(choice (const nil) string)
+  :group 'nnir)
+
 ;; Swish++.  Next three variables Copyright (C) 2000, 2001 Christoph
 ;; Conrad <christoph.conrad@gmx.de>.
 ;; Swish++ home page: http://homepage.mac.com/pauljlucas/software/swish/
@@ -959,6 +964,9 @@ (defun nnir-run-excite-search (query &op
 ;; send queries as literals
 ;; handle errors
 
+(eval-and-compile
+  (autoload 'mm-charset-to-coding-system "mm-util"))
+
 (defun nnir-run-imap (query &optional group)
   (require 'imap)
   (require 'nnimap)
@@ -975,13 +983,21 @@ (defun nnir-run-imap (query &optional gr
 	    (setq buf nnimap-server-buffer) ;; xxx
 	    (message "Searching %s..." group)
             (let ((arts 0)
-                  (mbx (gnus-group-real-name group)))
+                  (mbx (gnus-group-real-name group))
+		  coding-system)
               (when (imap-mailbox-select mbx nil buf)
                 (mapcar
                  (lambda (artnum)
                    (push (vector mbx artnum 1) artlist)
                    (setq arts (1+ arts)))
-                 (imap-search (concat "TEXT \"" qstring "\"") buf))
+		 (if (and nnir-imap-search-charset
+			  (setq coding-system (mm-charset-to-coding-system
+					       nnir-imap-search-charset)))
+		     (imap-search
+		      (concat "CHARSET " nnir-imap-search-charset " TEXT \""
+			      (mm-encode-coding-string qstring coding-system)
+			      "\"") buf)
+		   (imap-search (concat "TEXT \"" qstring "\"") buf)))
                 (message "Searching %s... %d matches" mbx arts)))
             (message "Searching %s...done" group))
         (quit nil))
